import type { CoreAnalytics } from '../analytics';
import type { CoreContext } from '../context';
export type PluginType = 'before' | 'after' | 'destination' | 'enrichment' | 'utility';
export interface CorePlugin<Ctx extends CoreContext = CoreContext, Analytics extends CoreAnalytics = any> {
    name: string;
    /**
     * A concatenation of the current destination name and the specific action. This field is only relevant for destination plugins.
     * @example ['Braze Web Mode (Actions) updateUserProfile'] // (and the plugin name would be 'Braze Web Mode (Actions)')
     */
    alternativeNames?: string[];
    /**
     * The version of the plugin.
     * @example '1.0.0'
     */
    version?: string;
    type: PluginType;
    isLoaded: () => boolean;
    load: (ctx: Ctx, instance: Analytics) => Promise<unknown>;
    unload?: (ctx: Ctx, instance: Analytics) => Promise<unknown> | unknown;
    ready?: () => Promise<unknown>;
    track?: (ctx: Ctx) => Promise<Ctx> | Ctx;
    identify?: (ctx: Ctx) => Promise<Ctx> | Ctx;
    page?: (ctx: Ctx) => Promise<Ctx> | Ctx;
    group?: (ctx: Ctx) => Promise<Ctx> | Ctx;
    alias?: (ctx: Ctx) => Promise<Ctx> | Ctx;
    screen?: (ctx: Ctx) => Promise<Ctx> | Ctx;
}
//# sourceMappingURL=index.d.ts.map